/*
 *  Authors: Jackie Liu <liuyun01@kylinos.cn>
 *
 *  Copyright (C) 2018,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef _MACHINE_TYPE_H_
#define _MACHINE_TYPE_H_

#include <linux/types.h>
#include <asm/phytium_platform.h>

void cpu_version_init(void);
bool phytium_check_cpu_type(u32 cpu_type);
const char *get_arm64_model_name(void);
/*
 * Phytium CPU VENDOR_ID defined as:
 * SIGNATURE_32 ('P','H','Y','T') in bios
 */
#define PHYTIUM_VENDOR_ID		0x54594850

/* Phytium 2000a4 */
#define PHYTIUM_CPU_2000_4_X		0x000300
#define PHYTIUM_CPU_2000_4		0x000301
#define PHYTIUM_CPU_2000_4_S		0x000302
#define PHYTIUM_CPU_2000_4_I		0x000303
#define PHYTIUM_CPU_2000_4_SI		0x000304
#define PHYTIUM_CPU_2000_4_2		0x000305
#define PHYTIUM_CPU_2000_4_I2		0x000306
#define PHYTIUM_CPU_2000_4_I1		0x000307
#define PHYTIUM_CPU_2000_4_M		0x000308
#define PHYTIUM_CPU_2000_4_EN		0x000311
#define PHYTIUM_CPU_2000_4_BC		0x000313
#define PHYTIUM_CPU_2000_4_ULP4C	0x000318

/* Phytium 2500 */
#define PHYTIUM_CPU_S2500_64		0x000400
#define PHYTIUM_CPU_S2500_64_C00	0x000401
#define PHYTIUM_CPU_S2500_64_I00	0x000402
#define PHYTIUM_CPU_S2500_56_C00	0x000403
#define PHYTIUM_CPU_S2500_56_I00	0x000404
#define PHYTIUM_CPU_S2500_48_C00	0x000405
#define PHYTIUM_CPU_S2500_48_I00	0x000406

/* Phytium D2000 */
#define PHYTIUM_CPU_D2000_8		0x000500
#define PHYTIUM_CPU_D2000_8_B8C		0x000501
#define PHYTIUM_CPU_D2000_8_E8C		0x000502
#define PHYTIUM_CPU_D2000_8_S8C		0x000503
#define PHYTIUM_CPU_D2000_8_EN8I	0x000504
#define PHYTIUM_CPU_D2000_4_S4C		0x000505
#define PHYTIUM_CPU_D2000_4_S4I		0x000506
#define PHYTIUM_CPU_D2000_8_ULP8C	0x000508
#define PHYTIUM_CPU_D2000_8_E8C2	0x000509
#define PHYTIUM_CPU_D2000_8_M8C		0x00050A
#define PHYTIUM_CPU_D2000_8_S8I		0x00050D

/* Phytium S5000 */
#define PHYTIUM_CPU_S5000		0x000600

/* Phytium E2000 */
#define PHYTIUM_CPU_E2000		0x000700

/* Phytium D3000 */
#define PHYTIUM_CPU_D3000		0x000900

#define is_ft1500a()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? is_part(MIDR_FT_1500A) : 0)
#define is_ft2000ahk()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? is_part(MIDR_FT_2000AHK) : 0)
#define is_ft2000plus()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? is_part(MIDR_FT_2000PLUS) : 0)

#define is_ft2000a4()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_2000_4_X) : 0)
#define is_ft2500()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_S2500_64) : 0)
/* for Phytium D2000 */
#define is_ftd2000()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_D2000_8) : 0)
/* for Phytium e2000 */
#define is_fte2000()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_E2000) : 0)
/* for Phytium D3000 */
#define is_ftd3000()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_D3000) : 0)
#define is_fts5000()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? phytium_check_cpu_type(PHYTIUM_CPU_S5000) : 0)

/* for Phytium D2000, FT2000a4 */
#define is_ft2000_pc()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? (is_part(MIDR_FT_2000A_4) && !is_ft2500()) : 0)
#define is_ft_all()	(IS_BUILTIN(CONFIG_ARCH_PHYTIUM) ? is_implementer(ARM_CPU_IMP_PHYTIUM) : 0)

#define is_kunpeng920()	(IS_BUILTIN(CONFIG_ARCH_HISI) ? is_part(MIDR_HISI_TSV110) : 0)
#define is_hisi()	(IS_BUILTIN(CONFIG_ARCH_HISI) ? is_implementer(ARM_CPU_IMP_HISI) : 0)

#endif
